/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.camel.batch

import org.apache.camel.Exchange
import org.apache.camel.CamelContext
import org.apache.camel.ConsumerTemplate
import metridoc.utils.AssertState
import org.apache.camel.impl.DefaultUnitOfWork

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 7/26/11
 * Time: 4:50 PM
 */
class ConsumeExchangeHandler {

    CamelContext camelContext
    String uri
    ConsumerTemplate consumerTemplate
    ProcessExchangeCallback callback
    int wait = -1

    CamelContext getCamelContext() {
        AssertState.notNull(camelContext, "the camelContext must be set for the ${ConsumeExchangeHandler.class} to work")
        return camelContext
    }

    ConsumerTemplate getConsumerTemplate() {
        if(consumerTemplate) {
            return consumerTemplate
        }

        consumerTemplate = getCamelContext().createConsumerTemplate()
        def uri = getUri()
        def endpoint = camelContext.getEndpoint(uri)
        consumerTemplate.getConsumerCache().getConsumer(endpoint) //this will start up the consumer

        return consumerTemplate
    }

    ProcessExchangeCallback getCallback() {
        AssertState.notNull(callback, "a ProcessExchangeCallback must be set for the ${ConsumeExchangeHandler.class} to work")
        return callback
    }

    String getUri() {
        AssertState.notEmpty(uri, "a uri must be set for ${ConsumeExchangeHandler.class}")
        return uri
    }

    Exchange next() {
        def consumer = getConsumerTemplate()
        def nextExchange
        switch (wait) {
            case 0:
                //get immediately
                nextExchange = consumer.receiveNoWait(uri)
                break;
            case -1:
                //wait forever
                nextExchange = consumer.receive(uri)
                break;
            default:
                //wait for specified time
                nextExchange = consumer.receive(uri, wait)
        }

        return nextExchange
    }

    /**
     * true if it ran, false if done (ie the returned exchange is null)
     * @return
     */
    boolean run() {
        def exchange = next()
        if(exchange == null) {
            return false
        }
        def work = new DefaultUnitOfWork(exchange)
        exchange.setUnitOfWork(work)
        Exception exception
        try {
            callback.process(exchange)
        } catch (Exception e) {
            exchange.setException(e)
            exception = e
        } finally {
            work.done(exchange) //deal with any rollbacks or completion stuff
        }

        if(exception) {
            throw exception
        }

        return true
    }
}
